@@ -2,7 +2,7 @@
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 from __future__ import annotations
 
-DOCUMENTATION = '''
+DOCUMENTATION = """
     name: ini
     version_added: "2.4"
     short_description: Uses an Ansible INI file as inventory source.
@@ -27,9 +27,9 @@ DOCUMENTATION = '''
         - Enabled in configuration by default.
         - Consider switching to YAML format for inventory sources to avoid confusion on the actual type of a variable.
           The YAML inventory plugin processes variable values consistently and correctly.
-'''
+"""
 
-EXAMPLES = '''# fmt: ini
+EXAMPLES = """# fmt: ini
 # Example 1
 [web]
 host1
@@ -70,10 +70,12 @@ host4
 [g2]
 host4 # same host as above, but member of 2 groups, will inherit vars from both
       # inventory hostnames are unique
-'''
+"""
 
 import ast
+import os
 import re
+import typing as t
 import warnings
 
 from ansible.inventory.group import to_safe_group_name
@@ -81,6 +83,7 @@ from ansible.plugins.inventory import BaseFileInventoryPlugin
 
 from ansible.errors import AnsibleError, AnsibleParserError
 from ansible.module_utils.common.text.converters import to_bytes, to_text
+from ansible._internal._datatag._tags import Origin, TrustedAsTemplate
 from ansible.utils.shlex import shlex_split
 
 
@@ -93,19 +96,23 @@ class InventoryModule(BaseFileInventoryPlugin):
     _COMMENT_MARKERS = frozenset((u';', u'#'))
     b_COMMENT_MARKERS = frozenset((b';', b'#'))
 
-    def __init__(self):
+    # template trust is applied internally to strings
+
+    def __init__(self) -> None:
 
         super(InventoryModule, self).__init__()
 
-        self.patterns = {}
-        self._filename = None
+        self.patterns: dict[str, re.Pattern] = {}
+        self._origin: Origin | None = None
 
-    def parse(self, inventory, loader, path, cache=True):
+    def verify_file(self, path):
+        # hardcode exclusion for TOML to prevent partial parsing of things we know we don't want
+        return super().verify_file(path) and os.path.splitext(path)[1] != '.toml'
+
+    def parse(self, inventory, loader, path: str, cache=True):
 
         super(InventoryModule, self).parse(inventory, loader, path)
 
-        self._filename = path
-
         try:
             # Read in the hosts, groups, and variables defined in the inventory file.
             if self.loader:
@@ -132,18 +139,24 @@ class InventoryModule(BaseFileInventoryPlugin):
                         # Non-comment lines still have to be valid uf-8
                         data.append(to_text(line, errors='surrogate_or_strict'))
 
-            self._parse(path, data)
-        except Exception as e:
-            raise AnsibleParserError(e)
+            self._origin = Origin(path=path, line_num=0)
+
+            try:
+                self._parse(data)
+            finally:
+                self._origin = self._origin.replace(line_num=None)
+
+        except Exception as ex:
+            raise AnsibleParserError('Failed to parse inventory.', obj=self._origin) from ex
 
     def _raise_error(self, message):
-        raise AnsibleError("%s:%d: " % (self._filename, self.lineno) + message)
+        raise AnsibleError(message)
 
-    def _parse(self, path, lines):
-        '''
+    def _parse(self, lines):
+        """
         Populates self.groups from the given array of lines. Raises an error on
         any parse failure.
-        '''
+        """
 
         self._compile_patterns()
 
@@ -155,9 +168,8 @@ class InventoryModule(BaseFileInventoryPlugin):
         pending_declarations = {}
         groupname = 'ungrouped'
         state = 'hosts'
-        self.lineno = 0
         for line in lines:
-            self.lineno += 1
+            self._origin = self._origin.replace(line_num=self._origin.line_num + 1)
 
             line = line.strip()
             # Skip empty lines and comments
@@ -189,7 +201,7 @@ class InventoryModule(BaseFileInventoryPlugin):
                     # declarations will take the appropriate action for a pending child group instead of
                     # incorrectly handling it as a var state pending declaration
                     if state == 'vars' and groupname not in pending_declarations:
-                        pending_declarations[groupname] = dict(line=self.lineno, state=state, name=groupname)
+                        pending_declarations[groupname] = dict(line=self._origin.line_num, state=state, name=groupname)
 
                     self.inventory.add_group(groupname)
 
@@ -229,7 +241,7 @@ class InventoryModule(BaseFileInventoryPlugin):
                 child = self._parse_group_name(line)
                 if child not in self.inventory.groups:
                     if child not in pending_declarations:
-                        pending_declarations[child] = dict(line=self.lineno, state=state, name=child, parents=[groupname])
+                        pending_declarations[child] = dict(line=self._origin.line_num, state=state, name=child, parents=[groupname])
                     else:
                         pending_declarations[child]['parents'].append(groupname)
                 else:
@@ -242,10 +254,11 @@ class InventoryModule(BaseFileInventoryPlugin):
         # We report only the first such error here.
         for g in pending_declarations:
             decl = pending_declarations[g]
+            self._origin = self._origin.replace(line_num=decl['line'])
             if decl['state'] == 'vars':
-                raise AnsibleError("%s:%d: Section [%s:vars] not valid for undefined group: %s" % (path, decl['line'], decl['name'], decl['name']))
+                raise ValueError(f"Section [{decl['name']}:vars] not valid for undefined group {decl['name']!r}.")
             elif decl['state'] == 'children':
-                raise AnsibleError("%s:%d: Section [%s:children] includes undefined group: %s" % (path, decl['line'], decl['parents'].pop(), decl['name']))
+                raise ValueError(f"Section [{decl['parents'][-1]}:children] includes undefined group {decl['name']!r}.")
 
     def _add_pending_children(self, group, pending):
         for parent in pending[group]['parents']:
@@ -255,10 +268,10 @@ class InventoryModule(BaseFileInventoryPlugin):
         del pending[group]
 
     def _parse_group_name(self, line):
-        '''
+        """
         Takes a single line and tries to parse it as a group name. Returns the
         group name if successful, or raises an error.
-        '''
+        """
 
         m = self.patterns['groupname'].match(line)
         if m:
@@ -267,10 +280,10 @@ class InventoryModule(BaseFileInventoryPlugin):
         self._raise_error("Expected group name, got: %s" % (line))
 
     def _parse_variable_definition(self, line):
-        '''
+        """
         Takes a string and tries to parse it as a variable definition. Returns
         the key and value if successful, or raises an error.
-        '''
+        """
 
         # TODO: We parse variable assignments as a key (anything to the left of
         # an '='"), an '=', and a value (anything left) and leave the value to
@@ -279,15 +292,15 @@ class InventoryModule(BaseFileInventoryPlugin):
 
         if '=' in line:
             (k, v) = [e.strip() for e in line.split("=", 1)]
-            return (k, self._parse_value(v))
+            return (self._origin.tag(k), self._parse_value(v))
 
         self._raise_error("Expected key=value, got: %s" % (line))
 
     def _parse_host_definition(self, line):
-        '''
+        """
         Takes a single line and tries to parse it as a host definition. Returns
         a list of Hosts if successful, or raises an error.
-        '''
+        """
 
         # A host definition comprises (1) a non-whitespace hostname or range,
         # optionally followed by (2) a series of key="some value" assignments.
@@ -312,14 +325,14 @@ class InventoryModule(BaseFileInventoryPlugin):
             if '=' not in t:
                 self._raise_error("Expected key=value host variable assignment, got: %s" % (t))
             (k, v) = t.split('=', 1)
-            variables[k] = self._parse_value(v)
+            variables[self._origin.tag(k)] = self._parse_value(v)
 
         return hostnames, port, variables
 
     def _expand_hostpattern(self, hostpattern):
-        '''
+        """
         do some extra checks over normal processing
-        '''
+        """
         # specification?
 
         hostnames, port = super(InventoryModule, self)._expand_hostpattern(hostpattern)
@@ -334,12 +347,31 @@ class InventoryModule(BaseFileInventoryPlugin):
 
         return (hostnames, port)
 
-    @staticmethod
-    def _parse_value(v):
-        '''
+    def _parse_recursive_coerce_types_and_tag(self, value: t.Any) -> t.Any:
+        if isinstance(value, str):
+            return TrustedAsTemplate().tag(self._origin.tag(value))
+        if isinstance(value, (list, tuple, set)):
+            # NB: intentional coercion of tuple/set to list, deal with it
+            return self._origin.tag([self._parse_recursive_coerce_types_and_tag(v) for v in value])
+        if isinstance(value, dict):
+            # FIXME: enforce keys are strings
+            return self._origin.tag({self._origin.tag(k): self._parse_recursive_coerce_types_and_tag(v) for k, v in value.items()})
+
+        if value is ...:  # literal_eval parses ellipsis, but it's not a supported variable type
+            value = TrustedAsTemplate().tag("...")
+
+        if isinstance(value, complex):  # convert unsupported variable types recognized by literal_eval back to str
+            value = TrustedAsTemplate().tag(str(value))
+
+        value = to_text(value, nonstring='passthru', errors='surrogate_or_strict')
+
+        return self._origin.tag(value)
+
+    def _parse_value(self, v: str) -> t.Any:
+        """
         Attempt to transform the string value from an ini file into a basic python object
         (int, dict, list, unicode string, etc).
-        '''
+        """
         try:
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore", SyntaxWarning)
@@ -352,13 +384,15 @@ class InventoryModule(BaseFileInventoryPlugin):
         except SyntaxError:
             # Is this a hash with an equals at the end?
             pass
-        return to_text(v, nonstring='passthru', errors='surrogate_or_strict')
+
+        # this is mostly unnecessary, but prevents the (possible) case of bytes literals showing up in inventory
+        return self._parse_recursive_coerce_types_and_tag(v)
 
     def _compile_patterns(self):
-        '''
+        """
         Compiles the regular expressions required to parse the inventory and
         stores them in self.patterns.
-        '''
+        """
 
         # Section names are square-bracketed expressions at the beginning of a
         # line, comprising (1) a group name optionally followed by (2) a tag
@@ -370,14 +404,14 @@ class InventoryModule(BaseFileInventoryPlugin):
         # [naughty:children] # only get coal in their stockings
 
         self.patterns['section'] = re.compile(
-            to_text(r'''^\[
+            to_text(r"""^\[
                     ([^:\]\s]+)             # group name (see groupname below)
                     (?::(\w+))?             # optional : and tag name
                 \]
                 \s*                         # ignore trailing whitespace
                 (?:\#.*)?                   # and/or a comment till the
                 $                           # end of the line
-            ''', errors='surrogate_or_strict'), re.X
+            """, errors='surrogate_or_strict'), re.X
         )
 
         # FIXME: What are the real restrictions on group names, or rather, what
@@ -386,10 +420,10 @@ class InventoryModule(BaseFileInventoryPlugin):
         # precise rules in order to support better diagnostics.
 
         self.patterns['groupname'] = re.compile(
-            to_text(r'''^
+            to_text(r"""^
                 ([^:\]\s]+)
                 \s*                         # ignore trailing whitespace
                 (?:\#.*)?                   # and/or a comment till the
                 $                           # end of the line
-            ''', errors='surrogate_or_strict'), re.X
+            """, errors='surrogate_or_strict'), re.X
         )
